
//
//  HTTPDataProcession.m
//  Unity-iPhone
//
//  Created by leon on 2021/2/2.
//

#import "HTTPDataProcession.h"

@interface HTTPDataProcession ()<NSURLSessionDataDelegate>
@end

@implementation HTTPDataProcession

#pragma mark - 网络请求数据
/// 向网络请求数据
+ (void)getHTTPDataProcession:(NSString*)urlString token:(NSString*)token success:(SuccessBlock)successBlock fail:(FailBlock)failBlock{
    
    // 1.创建url
    // 请求一个网页
//    NSString *urlString = @"http://test-shoes-api.hiyd.com/forum/friends?limit=1000";
    // 一些特殊字符编码
    urlString =  [urlString stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding];
    NSURL *url = [NSURL URLWithString:urlString];
    
    // 2.创建请求 并：设置缓存策略为每次都从网络加载 超时时间30秒
    NSURLRequest *request = [NSURLRequest requestWithURL:url cachePolicy:NSURLRequestReloadIgnoringLocalCacheData timeoutInterval:30];
    //2.创建一个 NSMutableURLRequest 添加 header
    NSMutableURLRequest *mutableRequest = [request mutableCopy];
    [mutableRequest addValue:token forHTTPHeaderField:@"token"];
    [mutableRequest addValue:@"application/x-www-form-urlencoded" forHTTPHeaderField:@"Content-Type"];
    //3.把值覆给request
    request = [mutableRequest copy];
    
    // 3.采用苹果提供的共享session
    NSURLSession *sharedSession = [NSURLSession sharedSession];

    // 4.由系统直接返回一个dataTask任务
    NSURLSessionDataTask *dataTask = [sharedSession dataTaskWithRequest:request completionHandler:^(NSData * _Nullable data, NSURLResponse * _Nullable response, NSError * _Nullable error){

        if (data && (error == nil)){
      
            // 网络请求完成之后就会执行，NSURLSession自动实现多线程
            successBlock(data);
            
        } else {//
            // 网络访问失败
            for (NSString * key  in error.userInfo){
                
                if ([key isEqualToString:@"com.alamofire.serialization.response.error.data"]){
                    
                    id errorObject = [NSJSONSerialization JSONObjectWithData:error.userInfo[key] options:1  error:nil];
                    
                    NSLog(@"错误信息 ERROR =========== >>> %@",error);
                    failBlock(errorObject);

                }
                
            }

        }
        
    }];
    // 5.每一个任务默认都是挂起的，需要调用 resume 方法
    [dataTask resume];
    
}

/// 向网络请求数据
+ (void)postHTTPDataProcession:(NSString *)urlString  withParams:(NSDictionary *)params token:(NSString*)token success:(SuccessBlock)successBlock fail:(FailBlock)failBlock{
    
      NSURL *url = [NSURL URLWithString:urlString];
      NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:url];
      request.HTTPMethod = @"POST";
      request.timeoutInterval = 10.0f;
      
      NSString *keyValueBody = [self getKeyValueStringWithDictionary:params];
      request.HTTPBody = [keyValueBody dataUsingEncoding:NSUTF8StringEncoding];
    
      NSURLSessionConfiguration *sessionConfiguration = [NSURLSessionConfiguration defaultSessionConfiguration];
      sessionConfiguration.HTTPAdditionalHeaders = @{@"Content-Type":@"application/x-www-form-urlencoded;charset=utf-8",@"token":token};
      
      NSURLSession *session = [NSURLSession sessionWithConfiguration:sessionConfiguration];
    
      NSURLSessionDataTask *dataTask= [session dataTaskWithRequest:request completionHandler:^(NSData * _Nullable data, NSURLResponse * _Nullable response, NSError * _Nullable error){
          
          if (data && (error == nil)){
        
              // 网络请求完成之后就会执行，NSURLSession自动实现多线程
              successBlock(data);
              
          }else{
              
              // 网络访问失败
              for (NSString * key  in error.userInfo){
                  
                  if ([key isEqualToString:@"com.alamofire.serialization.response.error.data"]){
                      
                      id errorObject = [NSJSONSerialization JSONObjectWithData:error.userInfo[key] options:1  error:nil];
                                            
                      failBlock(errorObject);

                  }
                  
              }

          }
          
      }];
      
      //发送请求
      [dataTask resume];
        
}

//
+(NSString*)getKeyValueStringWithDictionary:(NSDictionary*)dic{
    
    if (!dic||dic.count==0){
        return @"";
    }
    
    NSString *result = @"";
    NSArray *theAllKeys = [dic allKeys];
    for (int i=0; i<theAllKeys.count; i++){
        NSString *theKey = [theAllKeys objectAtIndex:i];
        NSString *theValue = [dic objectForKey:theKey];
        if (i==0){
            NSString *temp = [NSString stringWithFormat:@"%@=%@", theKey, theValue];
            result = [result stringByAppendingString:temp];
        }else {
            NSString *temp = [NSString stringWithFormat:@"&%@=%@", theKey, theValue];
            result = [result stringByAppendingString:temp];
        }
    }
    NSLog(@"result = %@",result);

    return result;
    
}


#pragma mark --------- request
+(void)getFriendsList:(UserFriendsDataBlock)userFriendsDataBlock{

    NSLog(@"gameFriends token ===>> %@ ",IOSSDK_TOKEN);

    [HTTPDataProcession getHTTPDataProcession:GAME_FRIENDS token:IOSSDK_TOKEN  success:^(id data){
            // 网络访问成功
//            NSLog(@"data=%@",[[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding]);
            NSError * error;
            NSDictionary *dictionary =[NSJSONSerialization JSONObjectWithData:data options:NSJSONReadingMutableLeaves error:&error];
        
//                NSLog(@"gameFriends dictionary === %@ ",dictionary);

            if (!error){
                
                int code = [dictionary[@"code"] intValue];

                if (code==0){//
                    
                    NSDictionary * dict = dictionary[@"data"];
                    NSArray * arr = dict[@"list"];
                    
                    NSMutableArray * userArr = [NSMutableArray new];//存储数据

                    for (NSDictionary * object in arr){
                        
                    NSDictionary * socialInfo = object[@"socialInfo"];
    //                    NSDictionary * dataDict = data[@"data"];
    //                    NSString * token = dataDict[@"token"];
    //                    NSLog(@"获取token值成功");
                    UserFriendsModel * model = [UserFriendsModel new];
                    model.code = 0;
                    [model setValuesForKeysWithDictionary:socialInfo];
                    
//                NSLog(@"model =========== >>> %d %@ %@ %d %@ %@",model.trans_id,model.nickname,model.city,model.gender,model.status,model.avatar);
                    
                    NSMutableDictionary * gameFriends = [NSMutableDictionary new];
                    [gameFriends setValue:model.status forKey:@"status"];
                    NSMutableDictionary * userInfo = [NSMutableDictionary new];
                    [userInfo setObject:[NSNumber numberWithInt:model.trans_id] forKey:@"id"];
                    [userInfo setValue:model.nickname forKey:@"nickname"];
                    [userInfo setObject:[NSNumber numberWithInt:model.gender] forKey:@"gender"];
                    [userInfo setValue:model.city forKey:@"city"];
                    [userInfo setValue:model.avatar forKey:@"avatar"];
                    [gameFriends setValue:userInfo forKey:@"user"];
                    
//                        NSLog(@"gameFriends =========== >>>%@",gameFriends);
                    [userArr addObject:gameFriends];
                    }
                    
                    NSData *jsonData = [NSJSONSerialization dataWithJSONObject:userArr options:NSJSONWritingPrettyPrinted error:nil];
                    
                    NSString * jsonString = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
                    
                    NSLog(@"GetUserFriends ==>> unity 请求好友列表  %@ ",jsonString);
                    
                    const char * cusChar =[jsonString UTF8String];
                    
                    userFriendsDataBlock(code,cusChar);
                 }
                
              }
            
            }fail:^(id errorDes){
                                    
    }];
    
}


+ (void)inviteFriends:(int)friendid info:(char*)info inviteDataBlock:(InviteDataBlock)inviteDataBlock{

    NSDictionary * jsonDict = [IOS_NSUSERDEFAULT objectForKey:IOSSDK_USERINFO];

    NSString * token = [jsonDict objectForKey:@"token"];
    NSString * invite_user_id = [NSString stringWithFormat:@"%d",friendid];
    NSString * invite = [NSString stringWithFormat:@"%s",info];
    NSString * game_id = jsonDict[@"game_id"];
    NSLog(@"InviteFriend ===>> token:%@  friendid:%d  info:%s",token,friendid,info);
    
    NSString *urlString = [NSString stringWithFormat:@"%@?invite_user_id=%@&invite=%@&game_id=%@",GAME_INVITE,invite_user_id,invite,game_id];
    [HTTPDataProcession getHTTPDataProcession:urlString token:token  success:^(id data){
        
            NSError * error;
            NSDictionary *dictionary =[NSJSONSerialization JSONObjectWithData:data options:NSJSONReadingMutableLeaves error:&error];
//                NSLog(@"gameFriends dictionary === %@ ",dictionary);

            if (!error){
                
                int code = [dictionary[@"code"] intValue];

                if (code==0){//
                    
                    inviteDataBlock(@"邀请成功");
                    
                 }
                
              }
            }fail:^(id errorDes){
    }];
    
}

+ (void)GetRank:(int)type rankDataBlock:(RankDataBlock)rankDataBlock{

    NSString * scoreType;
    if (type ==0){
        scoreType = @"world";
    }else{
        scoreType = @"friend";
    }
    
    NSString *urlString = [NSString stringWithFormat:@"%@&game_id=%@&scope=%@",GAME_RANK,IOSSDK_GAMEID,scoreType];

    NSLog(@"urlString == %@",urlString);
    
    [HTTPDataProcession getHTTPDataProcession:urlString token:IOSSDK_TOKEN  success:^(id data){

            NSError * error;
            NSDictionary *dictionary =[NSJSONSerialization JSONObjectWithData:data options:NSJSONReadingMutableLeaves error:&error];
        
//            NSLog(@"GetRank dictionary === %@ ",dictionary);
        
            if (!error){

                int code = [dictionary[@"code"] intValue];

                if (code==0){//

//                    NSLog(@"榜单数据 ====>> %@",dictionary);
                    
                    if (dictionary!=nil){
                        
                        int code = [dictionary[@"code"] intValue];

                        if (code==0){//
                            
                            NSDictionary * dict = dictionary[@"data"];
                            NSArray * arr = dict[@"records"];
                            
                            NSMutableArray * userArr = [NSMutableArray new];//存储数据
                            
                            NSMutableDictionary * list = [NSMutableDictionary new];

                            int rank_i = 0;
                            
                            for (NSDictionary * object in arr){
                                
                            rank_i++;
                                
                            RankModel * model = [RankModel new];
//                            model.code = 0;
                            [model setValuesForKeysWithDictionary:object];
                                                        
                            NSMutableDictionary * rank = [NSMutableDictionary new];

//                            [gameFriends setValue:model.status forKey:@"status"];
                            NSMutableDictionary * user = [NSMutableDictionary new];
                            [user setObject:[NSNumber numberWithInt:model.trans_id] forKey:@"id"];
                            [user setValue:model.user_name forKey:@"nickname"];
                            [user setObject:[NSNumber numberWithInt:model.user_gender] forKey:@"gender"];
                            [user setValue:@"" forKey:@"address"];
                            [user setValue:model.user_avatar forKey:@"avatar"];
                            
                            [rank setObject:user forKey:@"user"];
                            [rank setObject:[NSNumber numberWithInt:model.score] forKey:@"score"];
                            [rank setObject:[NSNumber numberWithInt:rank_i] forKey:@"rank"];
                                
                            [userArr addObject:rank];
                                
                            [list setObject:userArr forKey:@"list"];

                            }
                            
//                            NSLog(@"榜单数据 list ====>> %@",list);
                            
                            NSData *jsonData = [NSJSONSerialization dataWithJSONObject:list options:NSJSONWritingPrettyPrinted error:nil];
                            
                            NSString * jsonString = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
                            
//                            NSLog(@"HTTPDataProcession 请求榜单列表 GetRank 异步回调 ==>> %@ ",jsonString);
                            
                            const char * cusChar =[jsonString UTF8String];
                            
                            rankDataBlock(cusChar);

                          }

                      }

                  }

              }
        
            }fail:^(id errorDes){
               NSLog(@"HTTPDataProcession 获取榜单数据 错误 ====>> %@",errorDes);
    }];
    
}

/**
 开始游戏
 @param level 等级
*/
+(void)gameStart{

    NSString *urlString = [NSString stringWithFormat:@"%@?id=%@",GAME_START,IOSSDK_GAMEID];
    [HTTPDataProcession getHTTPDataProcession:urlString token:IOSSDK_TOKEN  success:^(id data){
        
            NSError * error;
            NSDictionary *dictionary =[NSJSONSerialization JSONObjectWithData:data options:NSJSONReadingMutableLeaves error:&error];
//                NSLog(@"gameFriends dictionary === %@ ",dictionary);

            if (!error){
                
                int code = [dictionary[@"code"] intValue];

                if (code==0){//
                    
                NSLog(@"gameStart 上传成功  ==>> %@",dictionary[@"message"]);
                    
                 }
                
              }
        
            }fail:^(id errorDes){
                
                NSLog(@"gameStart 上传失败  ==>> %@",errorDes);

    }];
    
}


/**
 结束游戏
 @param level 等级
*/
+(void)gameEnd{
    
    NSString *urlString = [NSString stringWithFormat:@"%@?id=%@",GAME_END,IOSSDK_GAMEID];
    [HTTPDataProcession getHTTPDataProcession:urlString token:IOSSDK_TOKEN  success:^(id data){
        
            NSError * error;
            NSDictionary *dictionary =[NSJSONSerialization JSONObjectWithData:data options:NSJSONReadingMutableLeaves error:&error];
//                NSLog(@"gameFriends dictionary === %@ ",dictionary);

            if (!error){
                
                int code = [dictionary[@"code"] intValue];

                if (code==0){//
                    
                    NSLog(@"gameEnd 上传成功  ==>> %@",dictionary[@"message"]);
                    
                 }
                
              }
        
            }fail:^(id errorDes){
                
                NSLog(@"gameEnd 上传失败  ==>> %@",errorDes);

    }];
    
}

+(void)postGameRecord:(int)level
                score:(double)score
               record:(int)record
                 mode:(int)mode
           opponentId:(int)opponentId
     gameEndDataBlock:(GameEndDataBlock)gameEndDataBlock{
    
    NSMutableDictionary * sportRecord = [NSMutableDictionary new];
    //缓存的token、game_id
    NSString * token = IOSSDK_TOKEN;
    NSString * game_id = IOSSDK_GAMEID;
    [sportRecord setObject:game_id forKey:@"game_id"];//游戏类型
    //预留字段:默认
    [sportRecord setObject:@0 forKey:@"consume"];//卡路里默认0
    [sportRecord setObject:@0 forKey:@"screen"];//默认0
    [sportRecord setObject:@0 forKey:@"distance"];//废弃
    [sportRecord setObject:@0 forKey:@"is_cancel"];//是否正常停止游戏 正常0 不正常1
    //unity回调数据
    [sportRecord setObject:[NSNumber numberWithInt:level] forKey:@"level"];
    [sportRecord setObject:[NSNumber numberWithDouble:score] forKey:@"score"];
    [sportRecord setObject:[NSNumber numberWithInt:record] forKey:@"record"];
    [sportRecord setObject:[NSNumber numberWithInt:mode] forKey:@"mode"];
    [sportRecord setObject:[NSNumber numberWithInt:opponentId] forKey:@"opponentId"];
   
   //新增加时间戳字段
    if ( [IOS_NSUSERDEFAULT objectForKey:IOSSDK_TIMESTAMP]!=nil){
        NSString * timestamp =  [IOS_NSUSERDEFAULT objectForKey:IOSSDK_TIMESTAMP];
        [sportRecord setObject:timestamp forKey:@"play_group"];
    }else{
        [sportRecord setObject:[AlgorithmTool returnTimestamp] forKey:@"play_group"];
    }
  
    //缓存的动作数据和持续时间
    NSDictionary * acheDict = [IOS_NSUSERDEFAULT objectForKey:IOSSDK_BLESDKDATA];
    if (acheDict!=nil){
        [sportRecord addEntriesFromDictionary:acheDict];
    }else{
        [sportRecord setObject:@0 forKey:@"jump"];
        [sportRecord setObject:@0 forKey:@"crouch"];
        [sportRecord setObject:@0 forKey:@"step"];
        [sportRecord setObject:@0 forKey:@"duration"];
    }
    
    NSLog(@"%@",sportRecord);

    [HTTPDataProcession postHTTPDataProcession:GAME_RECORD withParams:sportRecord  token:token success:^(id data){
           
        NSDictionary *dic = [NSJSONSerialization JSONObjectWithData:data options:NSJSONReadingMutableContainers error:nil];

        if (dic!=nil){

            int code = [dic[@"code"] intValue];

            if (code==0){
                
                NSLog(@"GameEnd 上传成功 %@",dic);
                gameEndDataBlock(@"上传成功");
                
        }else{
                NSLog(@"请求错误 =========== >>> %@",dic[@"msg"]);
             }
            
        }
        
    }fail:^(id errorDes){
        
        NSLog(@"请求失败 =========== >>> %@",errorDes);

    }];
    
}

@end
